------------------------------------------------------------------------------
-- $Id: rx_data_reg.vhd,v 1.1 2008/02/08 04:15:10 shurt Exp $
------------------------------------------------------------------------------
-- rx_data_reg.vhd
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
-- **                                                                       **
-- **  This text contains proprietary, confidential                         **
-- **  information of Xilinx, Inc. , is distributed by                      **
-- **  under license from Xilinx, Inc., and may be used,                    **
-- **  copied and/or disclosed only pursuant to the terms                   **
-- **  of a valid license agreement with Xilinx, Inc.                       **
-- **                                                                       **
-- **  Unmodified source code is guaranteed to place and route,             **
-- **  function and run at speed according to the datasheet                 **
-- **  specification. Source code is provided "as-is", with no              **
-- **  obligation on the part of Xilinx to provide support.                 **
-- **                                                                       **
-- **  Xilinx Hotline support of source code IP shall only include          **
-- **  standard level Xilinx Hotline support, and will only address         **
-- **  issues and questions related to the standard released Netlist        **
-- **  version of the core (and thus indirectly, the original core source). **
-- **                                                                       **
-- **  The Xilinx Support Hotline does not have access to source            **
-- **  code and therefore cannot answer specific questions related          **
-- **  to source HDL. The Xilinx Support Hotline will only be able          **
-- **  to confirm the problem in the Netlist version of the core.           **
-- **                                                                       **
-- **  This copyright and support notice must be retained as part           **
-- **  of this text at all times.                                           **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:        rx_data_reg.vhd
-- Version:         v3.00a
-- Description:     Intermediate register between WRPFIFO and the client fifo
--
------------------------------------------------------------------------------
-- Structure:   This section should show the hierarchical structure of the
--              designs. Separate lines with blank lines if necessary to improve
--              readability.
--
--              top_level.vhd
--                  -- second_level_file1.vhd
--                      -- third_level_file1.vhd
--                          -- fourth_level_file.vhd
--                      -- third_level_file2.vhd
--                  -- second_level_file2.vhd
--                  -- second_level_file3.vhd
--
--              This section is optional for common/shared modules but should
--              contain a statement stating it is a common/shared module.
------------------------------------------------------------------------------
-- Author:      DCW
-- History:
--  DCW      2004.06.09      -- First version
--
--  DRP      2006.05.03      -- Changed clock and reset names, 
--                           -- no functional changes made
--
--  <initials>      <date>
-- ^^^^^^
--      Description of changes. If multiple lines are needed to fully describe
--      the changes made to the design, these lines should align with each other.
-- ~~~~~~
--
--  <initials>      <date>
-- ^^^^^^
--      More changes
-- ~~~~~~
--
------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x"
--      reset signals:                          "rst", "rst_n"
--      generics:                               "C_*"
--      user defined types:                     "*_TYPE"
--      state machine next state:               "*_ns"
--      state machine current state:            "*_cs"
--      combinatorial signals:                  "*_cmb"
--      pipelined or register delay signals:    "*_d#"
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce"
--      internal version of : out   std_logic; port         "*_i"
--      device pins:                            "*_pin"
--      ports:                                  - Names begin with Uppercase
--      processes:                              "*_PROCESS"
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library unisim;
use unisim.vcomponents.all;

------------------------------------------------------------------------------
-- Port Declaration
------------------------------------------------------------------------------

entity rx_data_reg is
    generic (
             C_RESET_ACTIVE  : std_logic := '1';
             C_CLIENT_DWIDTH : integer   :=  8;
             C_BUS_DWIDTH    : integer   := 32
            );

    port    (
             LLTemac_Clk       : in  std_logic;
             LLTemac_Rst       : in  std_logic;
             RxD_Reg_CE        : in  std_logic;
             Partial_Wr        : in  std_logic;
             Rx_Cmplt          : in  std_logic;
             Cl_IF2RxD_Val_Ind : in  std_logic_vector(0 to (2*(C_CLIENT_DWIDTH/8)-1) );
             Cl_IF2Rx_Data     : in  std_logic_vector(0 to (2*C_CLIENT_DWIDTH-1) );
             IP2RFIFO_Data     : out std_logic_vector(0 to C_BUS_DWIDTH-1);
             IP2RFIFO_WrReq    : out std_logic
            );
end rx_data_reg;

------------------------------------------------------------------------------
-- Definition of Generics:
--
-- Definition of Ports:
--
------------------------------------------------------------------------------

architecture simulation of rx_data_reg is

begin

------------------------------------------------------------------------------
------------------------------------------------------------------------------
--                  GENERATE FOR BUS WIDTH 32, CLIENT WIDTH 8 
--
--               Client width doubles, 16 --> 32, 2 enable bits
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

G_32_8 : if ((C_BUS_DWIDTH = 32) and (C_CLIENT_DWIDTH = 8)) generate

    --------------------------------------------------------------------------
    -- Signal Declarations
    --------------------------------------------------------------------------
    
    signal rxD_Reg_CE_i    : std_logic_vector(0 to 1);
    signal iP2RFIFO_Data_i : std_logic_vector(0 to C_BUS_DWIDTH-1);
    signal cl_If2Rx_Data_i : std_logic_vector(0 to (2*C_CLIENT_DWIDTH-1));
    
    begin

    --------------------------------------------------------------------------
    -- Concurrent assignments
    --------------------------------------------------------------------------

    IP2RFIFO_Data <= iP2RFIFO_Data_i;
    
    --------------------------------------------------------------------------
    -- This process registers Cl_If2Rx_Data input
    --------------------------------------------------------------------------    
    
    CL_IF2RX_DATA_REG : process(LLTemac_Clk)
    begin
      if(rising_edge(LLTemac_Clk)) then
         if(LLTemac_Rst='1') then
            cl_If2Rx_Data_i <= (others=>'0');
         else
            cl_If2Rx_Data_i <= Cl_If2Rx_Data;
         end if;
      end if;
    end process;
    
    --------------------------------------------------------------------------
    -- RXD_REG_CE_PROCESS
    --------------------------------------------------------------------------
    
    RXD_REG_CE_PROCESS : process (LLTemac_Clk)
    begin
    
        if (LLTemac_Clk'event and LLTemac_Clk = '1') then
            if ((LLTemac_Rst = C_RESET_ACTIVE) or (Rx_Cmplt = '1')) then
                rxD_Reg_CE_i <= "00";
            elsif (RxD_Reg_CE = '1') then
                if    (rxD_Reg_CE_i  = "00") then
                       rxD_Reg_CE_i <= "10";
                elsif (rxD_Reg_CE_i  = "10") then
                       rxD_Reg_CE_i <= "01";
                elsif (rxD_Reg_CE_i  = "01") then
                       rxD_Reg_CE_i <= "10";
                else
                    rxD_Reg_CE_i <= "00";
                end if;
            end if;
        end if;
    end process;

    --------------------------------------------------------------------------
    -- RX_DATA_REG_PROCESS
    --------------------------------------------------------------------------
    
    RX_DATA_REG_PROCESS : process (LLTemac_Clk)
    begin
    
        if (LLTemac_Clk'event and LLTemac_Clk = '1') then
            if (LLTemac_Rst = C_RESET_ACTIVE) then
                iP2RFIFO_Data_i <= (others => '0');
            else                
                if (rxD_Reg_CE_i = "10") then
                    iP2RFIFO_Data_i(0  to 15) <= Cl_IF2Rx_Data_i;
                    iP2RFIFO_Data_i(16 to 31) <= (others => '0');
                elsif (rxD_Reg_CE_i = "01") then
                    iP2RFIFO_Data_i(0  to 15) <= iP2RFIFO_Data_i(0 to 15);
                    iP2RFIFO_Data_i(16 to 31) <= Cl_IF2Rx_Data_i;--iP2RFIFO_Data_i(16 to 31);             
                end if;
            end if;
        end if;
    end process;

    --------------------------------------------------------------------------
    -- IP2RFIFO_WRREQ_PROCESS
    --------------------------------------------------------------------------
    
    IP2RFIFO_WRREQ_PROCESS : process (LLTemac_Clk)
    begin
    
        if (LLTemac_Clk'event and LLTemac_Clk = '1') then
            if (LLTemac_Rst = C_RESET_ACTIVE) then
                IP2RFIFO_WrReq <= '0';
            elsif ( (RxD_Reg_CE = '1') and (rxD_Reg_CE_i <= "01") ) then
                IP2RFIFO_WrReq <= rxD_Reg_CE_i(1);
          elsif (Partial_Wr = '1') then
              IP2RFIFO_WrReq <= '1';
            else
                IP2RFIFO_WrReq <= '0';
            end if;
        end if;
    end process;

end generate;

------------------------------------------------------------------------------
------------------------------------------------------------------------------
--                  GENERATE FOR BUS WIDTH 32, CLIENT WIDTH 16 
--
--               Client width doubles, 32 --> 32, 1 enable bit
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

--G_32_16 : if ((C_BUS_DWIDTH = 32) and (C_CLIENT_DWIDTH = 16)) generate

    --------------------------------------------------------------------------
    -- Signal Declarations
    --------------------------------------------------------------------------
    
    --signal iP2RFIFO_Data_i : std_logic_vector(0 to C_BUS_DWIDTH-1);
    
    --begin

    --------------------------------------------------------------------------
    -- Concurrent assignments
    --------------------------------------------------------------------------

    --IP2RFIFO_Data <= iP2RFIFO_Data_i;

    --------------------------------------------------------------------------
    -- RX_DATA_REG_PROCESS
    --------------------------------------------------------------------------
    
    --RX_DATA_REG_PROCESS : process (LLTemac_Clk)
    --begin
    --
    --    if (LLTemac_Clk'event and LLTemac_Clk = '1') then
    --        if (LLTemac_Rst = C_RESET_ACTIVE) then
    --            iP2RFIFO_Data_i <= (others => '0');
    --        else
    --            if (RxD_Reg_CE = '1') then
    --                iP2RFIFO_Data_i <= Cl_IF2Rx_Data;
    --            end if;
    --        end if;
    --    end if;
    --end process;


    --------------------------------------------------------------------------
    -- IP2RFIFO_WRREQ_PROCESS
    --------------------------------------------------------------------------
    
    --IP2RFIFO_WRREQ_PROCESS : process (LLTemac_Clk)
    --begin
    --
    --    if (LLTemac_Clk'event and LLTemac_Clk = '1') then
    --        if (LLTemac_Rst = C_RESET_ACTIVE) then
    --            IP2RFIFO_WrReq <= '0';
    --        elsif (RxD_Reg_CE = '1')then
    --            IP2RFIFO_WrReq <= '1';
--  --        elsif (Partial_Wr = '1') then
--  --            IP2RFIFO_WrReq <= '1';
    --        else
    --            IP2RFIFO_WrReq <= '0';
    --        end if;
    --    end if;
    --end process;

--end generate;

------------------------------------------------------------------------------
------------------------------------------------------------------------------
--                  GENERATE FOR BUS WIDTH 64, CLIENT WIDTH 8 
--
--               Client width doubles, 16 --> 64, 4 enable states
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

--G_64_8 : if ((C_BUS_DWIDTH = 64) and (C_CLIENT_DWIDTH = 8)) generate

    --------------------------------------------------------------------------
    -- Signal Declarations
    --------------------------------------------------------------------------
    
    --signal rxD_Reg_CE_i        : std_logic_vector(0 to 1);
    --signal iP2RFIFO_Data_i     : std_logic_vector(0 to C_BUS_DWIDTH-1);
    --signal partial_remain      : std_logic;
    --signal cl_IF2Rx_Data_i     : std_logic_vector(0 to 15);
    
    --begin

    --------------------------------------------------------------------------
    -- Concurrent assignments
    --------------------------------------------------------------------------

    --IP2RFIFO_Data <= iP2RFIFO_Data_i;

    --------------------------------------------------------------------------
    -- CL_DATA_ENABKE_PROCESS
    --------------------------------------------------------------------------
    
    --CL_DATA_ENABKE_PROCESS : process (Cl_IF2Rx_Data, Cl_IF2RxD_Val_Ind)
    --begin
    --    if    (Cl_IF2RxD_Val_Ind = "11") then cl_IF2Rx_Data_i <= Cl_IF2Rx_Data;
    --    elsif (Cl_IF2RxD_Val_Ind = "10") then cl_IF2Rx_Data_i <= Cl_IF2Rx_Data(0 to 7) & X"00";
    --    elsif (Cl_IF2RxD_Val_Ind = "01") then cl_IF2Rx_Data_i <= X"00" & Cl_IF2Rx_Data(8 to 15);
    --    else
    --        cl_IF2Rx_Data_i <= X"0000";
    --    end if;
    --end process;

    --------------------------------------------------------------------------
    -- JOHNSON_CNTR_PROCESS
    --------------------------------------------------------------------------
    
    --JOHNSON_CNTR_PROCESS : process (LLTemac_Clk)
    --begin
    
    --    if (LLTemac_Clk'event and LLTemac_Clk = '1') then
    --        if ( (LLTemac_Rst = C_RESET_ACTIVE) or (Rx_Cmplt = '1') ) then
    --            rxD_Reg_CE_i <= "00";
    --        elsif (RxD_Reg_CE = '1') then
    --            rxD_Reg_CE_i <= not rxD_Reg_CE_i(1) & rxD_Reg_CE_i(0);
    --        end if;
    --    end if;
    --end process;

    --------------------------------------------------------------------------
    -- RX_DATA_REG_PROCESS
    --------------------------------------------------------------------------
    
    --RX_DATA_REG_PROCESS : process (LLTemac_Clk)
    --begin
    
    --    if (LLTemac_Clk'event and LLTemac_Clk = '1') then
    --        if (LLTemac_Rst = C_RESET_ACTIVE) then
    --            iP2RFIFO_Data_i <= (others => '0');
    --        elsif (RxD_Reg_CE = '1') then
    --            if (rxD_Reg_CE_i = "00") then
    --                iP2RFIFO_Data_i(0  to 15) <= cl_IF2Rx_Data_i;
    --                iP2RFIFO_Data_i(16 to 31) <= (others => '0');
    --                iP2RFIFO_Data_i(32 to 47) <= (others => '0');
    --                iP2RFIFO_Data_i(48 to 63) <= (others => '0');
    --            elsif (rxD_Reg_CE_i = "10") then
    --                iP2RFIFO_Data_i(0  to 15) <= iP2RFIFO_Data_i(0  to 15);
    --                iP2RFIFO_Data_i(16 to 31) <= cl_IF2Rx_Data_i;
    --                iP2RFIFO_Data_i(32 to 47) <= (others => '0');
    --                iP2RFIFO_Data_i(48 to 63) <= (others => '0');
    --            elsif (rxD_Reg_CE_i = "11") then
    --                iP2RFIFO_Data_i(0  to 15) <= iP2RFIFO_Data_i(0  to 15);
    --                iP2RFIFO_Data_i(16 to 31) <= iP2RFIFO_Data_i(16 to 31);
    --                iP2RFIFO_Data_i(32 to 47) <= cl_IF2Rx_Data_i;
    --                iP2RFIFO_Data_i(48 to 63) <= (others => '0');
    --            elsif (rxD_Reg_CE_i = "01") then
    --                iP2RFIFO_Data_i(0  to 15) <= iP2RFIFO_Data_i(0  to 15);
    --                iP2RFIFO_Data_i(16 to 31) <= iP2RFIFO_Data_i(16 to 31);
    --                iP2RFIFO_Data_i(32 to 47) <= iP2RFIFO_Data_i(32 to 47);
    --                iP2RFIFO_Data_i(48 to 63) <= cl_IF2Rx_Data_i;
    --            end if;
    --        end if;
    --    end if;
    --end process;

    --------------------------------------------------------------------------
    -- IP2RFIFO_WRREQ_PROCESS
    --------------------------------------------------------------------------
    
    --IP2RFIFO_WRREQ_PROCESS : process (LLTemac_Clk)
    --begin
    --
    --    if (LLTemac_Clk'event and LLTemac_Clk = '1') then
    --        if (LLTemac_Rst = C_RESET_ACTIVE) then
    --            IP2RFIFO_WrReq <= '0';
    --        elsif ( (RxD_Reg_CE = '1') and (rxD_Reg_CE_i = "01") ) then
    --            IP2RFIFO_WrReq <= '1';
    --        elsif ( (Partial_Wr = '1') and (partial_remain = '1') ) then
    --            IP2RFIFO_WrReq <= '1';
    --        else
    --            IP2RFIFO_WrReq <= '0';
    --        end if;
    --    end if;
    --end process;

    --------------------------------------------------------------------------
    -- PARTIAL_REMAIN_PROCESS
    --------------------------------------------------------------------------
    
    --PARTIAL_REMAIN_PROCESS : process (LLTemac_Clk)
    --begin
    --
    --    if (LLTemac_Clk'event and LLTemac_Clk = '1') then
    --        if (LLTemac_Rst = C_RESET_ACTIVE) then
    --            partial_remain <= '0';
    --        elsif ( (RxD_Reg_CE = '1') and (rxD_Reg_CE_i = "00") ) then
    --            partial_remain <= '1';
    --        elsif ( (RxD_Reg_CE = '1') and (rxD_Reg_CE_i = "01") ) then
    --            partial_remain <= '0';
    --        end if;
    --    end if;
    --end process;

--end generate;

------------------------------------------------------------------------------
------------------------------------------------------------------------------
--                  GENERATE FOR BUS WIDTH 64, CLIENT WIDTH 16 
--
--               Client width doubles, 32 --> 64, 2 enable bits
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------

--G_64_16 : if ((C_BUS_DWIDTH = 32) and (C_CLIENT_DWIDTH = 16)) generate

    --------------------------------------------------------------------------
    -- Signal Declarations
    --------------------------------------------------------------------------
    
    --signal rxD_Reg_CE_i    : std_logic_vector(0 to 1);
    --signal iP2RFIFO_Data_i : std_logic_vector(0 to C_BUS_DWIDTH-1);
    
    --begin

    --------------------------------------------------------------------------
    -- Concurrent assignments
    --------------------------------------------------------------------------

    --IP2RFIFO_Data <= iP2RFIFO_Data_i;

    --------------------------------------------------------------------------
    -- RXD_REG_CE_PROCESS
    --------------------------------------------------------------------------
    
    --RXD_REG_CE_PROCESS : process (LLTemac_Clk)
    --begin
    
    --    if (LLTemac_Clk'event and LLTemac_Clk = '1') then
    --        if (LLTemac_Rst = C_RESET_ACTIVE) then
    --            rxD_Reg_CE_i <= "00";
    --        elsif (RxD_Reg_CE = '1') then
    --            if (rxD_Reg_CE_i = "00") then
    --                rxD_Reg_CE_i <= "10";
    --            elsif (rxD_Reg_CE_i = "10") then
    --                rxD_Reg_CE_i <= "01";
    --            elsif (rxD_Reg_CE_i = "01") then
    --                rxD_Reg_CE_i <= "10";
    --            else
    --                rxD_Reg_CE_i <= "00";
    --            end if;
    --        end if;
    --    end if;
    --end process;

    --------------------------------------------------------------------------
    -- RX_DATA_REG_PROCESS
    --------------------------------------------------------------------------
    
    --RX_DATA_REG_PROCESS : process (LLTemac_Clk)
    --begin
    
    --    if (LLTemac_Clk'event and LLTemac_Clk = '1') then
    --        if (LLTemac_Rst = C_RESET_ACTIVE) then
    --            iP2RFIFO_Data_i <= (others => '0');
    --        else
    --            if (rxD_Reg_CE_i = "10") then
    --                iP2RFIFO_Data_i(0  to 15) <= Cl_IF2Rx_Data;
    --                iP2RFIFO_Data_i(16 to 31) <= (others => '0');
    --            elsif (rxD_Reg_CE_i = "01") then
    --                iP2RFIFO_Data_i(0  to 15) <= iP2RFIFO_Data_i(0  to 15);
    --                iP2RFIFO_Data_i(16 to 31) <= iP2RFIFO_Data_i;
    --            end if;
    --        end if;
    --    end if;
    --end process;

    --------------------------------------------------------------------------
    -- IP2RFIFO_WRREQ_PROCESS
    --------------------------------------------------------------------------
    
    --IP2RFIFO_WRREQ_PROCESS : process (LLTemac_Clk)
    --begin
    --
    --    if (LLTemac_Clk'event and LLTemac_Clk = '1') then
    --        if (LLTemac_Rst = C_RESET_ACTIVE) then
    --            IP2RFIFO_WrReq <= '0';
    --        elsif ( (RxD_Reg_CE = '1') and (rxD_Reg_CE_i <= "01") ) then
    --            IP2RFIFO_WrReq <= rxD_Reg_CE_i(1);
--  --        elsif (Partial_Wr = '1') then
--  --            IP2RFIFO_WrReq <= '1';
    --        else
    --            IP2RFIFO_WrReq <= '0';
    --        end if;
    --    end if;
    --end process;

--end generate;

end simulation;
