------------------------------------------------------------------------------
-- $Id: rx_llink_top.vhd,v 1.3 2008/06/19 21:41:00 mwelter Exp $
------------------------------------------------------------------------------
-- rx.vhd
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
-- **                                                                       **
-- **  This text contains proprietary, confidential                         **
-- **  information of Xilinx, Inc. , is distributed by                      **
-- **  under license from Xilinx, Inc., and may be used,                    **
-- **  copied and/or disclosed only pursuant to the terms                   **
-- **  of a valid license agreement with Xilinx, Inc.                       **
-- **                                                                       **
-- **  Unmodified source code is guaranteed to place and route,             **
-- **  function and run at speed according to the datasheet                 **
-- **  specification. Source code is provided "as-is", with no              **
-- **  obligation on the part of Xilinx to provide support.                 **
-- **                                                                       **
-- **  Xilinx Hotline support of source code IP shall only include          **
-- **  standard level Xilinx Hotline support, and will only address         **
-- **  issues and questions related to the standard released Netlist        **
-- **  version of the core (and thus indirectly, the original core source). **
-- **                                                                       **
-- **  The Xilinx Support Hotline does not have access to source            **
-- **  code and therefore cannot answer specific questions related          **
-- **  to source HDL. The Xilinx Support Hotline will only be able          **
-- **  to confirm the problem in the Netlist version of the core.           **
-- **                                                                       **
-- **  This copyright and support notice must be retained as part           **
-- **  of this text at all times.                                           **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:        rx_llink_top.vhd
-- Version:         v1.00a
-- Description:     Receive interface between LL and hard Temac
--
------------------------------------------------------------------------------
-- Structure:   This section should show the hierarchical structure of the
--              designs. Separate lines with blank lines if necessary to improve
--              readability.
--
--              top_level.vhd
--                  -- second_level_file1.vhd
--                      -- third_level_file1.vhd
--                          -- fourth_level_file.vhd
--                      -- third_level_file2.vhd
--                  -- second_level_file2.vhd
--                  -- second_level_file3.vhd
--
--              This section is optional for common/shared modules but should
--              contain a statement stating it is a common/shared module.
------------------------------------------------------------------------------
-- Author:      DRP
-- History:
--  DRP      2006.05.18      -- First version
--
--  <initials>      <date>
-- ^^^^^^
--      Description of changes. If multiple lines are needed to fully describe
--      the changes made to the design, these lines should align with each other.
-- ~~~~~~
--
--  <initials>      <date>
-- ^^^^^^
--      More changes
-- ~~~~~~
--
------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x"
--      reset signals:                          "rst", "rst_n"
--      generics:                               "C_*"
--      user defined types:                     "*_TYPE"
--      state machine next state:               "*_ns"
--      state machine current state:            "*_cs"
--      combinatorial signals:                  "*_cmb"
--      pipelined or register delay signals:    "*_d#"
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce"
--      internal version of : out   std_logic; port         "*_i"
--      device pins:                            "*_pin"
--      ports:                                  - Names begin with Uppercase
--      processes:                              "*_PROCESS"
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
------------------------------------------------------------------------------

------------------------------------------------------------------------------
-- Libraries used;
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_arith.conv_std_logic_vector;
use ieee.numeric_std.all;    
use ieee.std_logic_misc.all;

library proc_common_v2_00_a;
use proc_common_v2_00_a.coregen_comp_defs.all;
use proc_common_v2_00_a.proc_common_pkg.log2;

-- synopsys translate_off
library XilinxCoreLib;
-- synopsys translate_on


library xps_ll_temac_v1_01_b;
use xps_ll_temac_v1_01_b.all;

library unisim;
use unisim.vcomponents.all;

------------------------------------------------------------------------------
-- Definition of Generics:
--
-- Definition of Ports:
--
------------------------------------------------------------------------------

entity rx_llink_top is
    generic (
             C_FAMILY        : string    := "virtex5";  
             C_RESET_ACTIVE  : std_logic :=  '1';
             C_BUS_DWIDTH    : integer   :=   32;
             C_TEMAC_TYPE    : integer   :=    0;  
               -- 0 - Virtex 5 hard TEMAC (FX, LXT, SXT devices)                
               -- 1 - Virtex 4 hard TEMAC (FX)               
               -- 2 - Soft TEMAC         
             C_TEMAC_RXCSUM  : integer   :=    0;
             C_CLIENT_DWIDTH : integer   :=    8;
             C_TEMAC_RXFIFO  : integer   := 4096 
            );

    port    (
             LLTemac_Clk              : in  std_logic;
             LLTemac_Rst              : in  std_logic;
             TemacLL_SOF_n            : out std_logic;
             TemacLL_SOP_n            : out std_logic;
             TemacLL_EOF_n            : out std_logic;
             TemacLL_EOP_n            : out std_logic;
             TemacLL_SRC_RDY_n        : out std_logic;
             TemacLL_DST_RDY_n        : in  std_logic;
             TemacLL_REM              : out std_logic_vector(0 to 3);
             TemacLL_Data             : out std_logic_vector(0 to 31);
             
             RegCR_BrdCast_Rej        : in  std_logic;
             RegCR_MulCast_Rej        : in  std_logic;

             Rx_pckt_rej              : out std_logic;
             Rx_cmplt                 : out std_logic;
             Pckt_Ovr_Run             : out std_logic;

             Rx_Cl_Clk                : in  std_logic;
             EmacClientRxBadFrame     : in  std_logic;
             EmacClientRxd            : in  std_logic_vector(C_CLIENT_DWIDTH-1 downto 0);
             EmacClientRxdVld         : in  std_logic;
             EmacClientRxFrameDrop    : in  std_logic;
             EmacClientRxGoodFrame    : in  std_logic;
             EmacClientRxStats        : in  std_logic_vector(6 downto 0);
             SoftEmacClientRxStats    : in  std_logic_vector(27 downto 0);
             EmacClientRxStatsVld     : in  std_logic;
--             TakeRxStatsNow           : in  std_logic;
             EmacClientRxStats_i      : in  std_logic_vector(6 downto 0);
             ClFifoOverflow           : out std_logic
            );
end rx_llink_top;

------------------------------------------------------------------------------
-- Architecture
------------------------------------------------------------------------------

architecture beh of rx_llink_top is

attribute KEEP : string;
attribute KEEP of Rx_Cl_Clk : signal is "TRUE";

------------------------------------------------------------------------------
-- Constant Declarations
------------------------------------------------------------------------------

constant RX_LEN_LENGTH : integer := 14;
constant C_FIFODEPTH   : integer := (log2(C_TEMAC_RXFIFO/4))-1;

------------------------------------------------------------------------------
-- Signal Declarations
------------------------------------------------------------------------------

signal rsFIFO2LL_Empty       : std_logic;
signal rsFIFO2LL_Und         : std_logic;
signal rsFIFO2LL_Rdack       : std_logic;
signal lL2RSFIFO_Rden        : std_logic;
signal rsFIFO2LL_Data        : std_logic_vector(0 to 35);
signal rxFIFO2LL_Empty       : std_logic;
signal rxFIFO2LL_Und         : std_logic;
signal rxFIFO2LL_Rdack       : std_logic;
signal lL2RXFIFO_Rden        : std_logic;
signal rxFIFO2LL_Data        : std_logic_vector(0 to 35);
signal rsFIFO2IP_Wrack       : std_logic;
signal rsFIFO2IP_Full        : std_logic;
signal rsFIFO2IP_Ovr         : std_logic;
signal ip2RSFIFO_Wreq        : std_logic;
signal ip2RSFIFO_Data        : std_logic_vector(0 to 35);
signal rxFIFO2IP_Wrack       : std_logic;
signal rxFIFO2IP_Almost_Full : std_logic;
signal rxFIFO2IP_Full        : std_logic;
signal rxFIFO2IP_Ovr         : std_logic; 
signal ip2RXFIFO_Wreq        : std_logic;
signal ip2RXFIFO_Data        : std_logic_vector(0 to 35);
signal full_d                : std_logic;
signal empty_d               : std_logic;
signal afull_d               : std_logic;
signal afull_d1              : std_logic;
signal afull_d2              : std_logic;
signal afull_d3              : std_logic;
signal afull_d4              : std_logic;
signal afull_d5              : std_logic;
signal afull_d6              : std_logic;
signal aempty_d              : std_logic;
signal full_c                : std_logic;
signal empty_c               : std_logic;
signal afull_c               : std_logic;
signal aempty_c              : std_logic;
signal occupancy_i           : std_logic_vector(0 to C_FIFODEPTH);
signal fullMask              : std_logic_vector(0 to C_FIFODEPTH);
signal emptyMask             : std_logic_vector(0 to C_FIFODEPTH);
signal afullMask             : std_logic_vector(0 to C_FIFODEPTH);
signal aemptyMask            : std_logic_vector(0 to C_FIFODEPTH);
signal ip2RXFIFO_Wreq_unqualified : std_logic;
signal resetFifosOutOfSync : std_logic;
signal comboFifoReset      : std_logic;

signal localLinkPaused       : std_logic;
signal pauseLocalLinkFull    : std_logic;
signal fifoFullReset         : std_logic;
signal rx_cmplt_i            : std_logic;

begin
  Rx_cmplt     <= (rx_cmplt_i);        
--  Rx_cmplt     <= (not(rsFIFO2LL_Empty) or rx_cmplt_i);        
  comboFifoReset <= resetFifosOutOfSync or LLTemac_Rst or fifoFullReset;

  GENMASK: for I in C_FIFODEPTH downto 0 generate
    fullMask(I) <= '1';
    emptyMask(I) <= '0';
  end generate;
  afullMask <= fullMask-2;
  aemptyMask <= emptyMask+1;

------------------------------------------------------------------------------
-- Concurrent Signal Assignments
------------------------------------------------------------------------------


--rxFIFO2IP_Ovr <= ip2RXFIFO_Wreq and rxFIFO2IP_full;
rxFIFO2IP_Ovr <= ip2RXFIFO_Wreq_unqualified and (rxFIFO2IP_full or afull_c or afull_d or afull_d1 or afull_d2 or afull_d3 or afull_d4 or afull_d5 or afull_d6);
rxFIFO2LL_Und <= rxFIFO2LL_Empty and lL2RXFIFO_Rden;
rsFIFO2IP_Ovr <= rsFIFO2IP_Full and ip2RSFIFO_Wreq;
rsFIFO2LL_Und <= rSFIFO2LL_Empty and lL2RSFIFO_Rden;



I_RX_LL_IF : entity xps_ll_temac_v1_01_b.rx_ll_if 
   generic map(
      C_RESET_ACTIVE    => C_RESET_ACTIVE,
      C_BUS_DWIDTH      => C_BUS_DWIDTH,
      C_CLIENT_DWIDTH   => C_CLIENT_DWIDTH,
      C_RX_LEN_LENGTH   => RX_LEN_LENGTH
      )

   port map(
      LocalLinkPaused         => localLinkPaused,      -- out
      PauseLocalLinkFull      => pauseLocalLinkFull,   -- in 
      FifoFullReset           => fifoFullReset,        -- in 
      ResetFifosOutOfSync     => resetFifosOutOfSync,  -- out
      LLTemac_Clk             => LLTemac_Clk,          -- in  
      LLTemac_Rst             => LLTemac_Rst,          -- in  
      RSFIFO2LL_Empty         => rSFIFO2LL_Empty,      -- in  
      RSFIFO2LL_Und           => rSFIFO2LL_Und,        -- in  
      RSFIFO2LL_Rdack         => rSFIFO2LL_Rdack,      -- in  
      lL2RSFIFO_Rden          => lL2RSFIFO_Rden,       -- out 
      RSFIFO2LL_Data          => rsFIFO2LL_Data,       -- in  
      RXFIFO2LL_Empty         => rxFIFO2LL_Empty,      -- in  
      RXFIFO2LL_Und           => rxFIFO2LL_Und,        -- in  
      RXFIFO2LL_Rdack         => rxFIFO2LL_Rdack,      -- in  
      lL2RXFIFO_Rden          => lL2RXFIFO_Rden,       -- out 
      RXFIFO2LL_Data          => rxFIFO2LL_Data,       -- in   
      TemacLL_SOF_n           => TemacLL_SOF_n,        -- out 
      TemacLL_SOP_n           => TemacLL_SOP_n,        -- out 
      TemacLL_EOF_n           => TemacLL_EOF_n,        -- out 
      TemacLL_EOP_n           => TemacLL_EOP_n,        -- out 
      TemacLL_SRC_RDY_n       => TemacLL_SRC_RDY_n,    -- out 
      TemacLL_DST_RDY_n       => TemacLL_DST_RDY_n,    -- in  
      TemacLL_REM             => TemacLL_REM,          -- out 
      TemacLL_Data            => TemacLL_Data          -- out 
      );

 I_RX_STATUS_FIFO : sync_fifo_v5_0
    generic map(
              c_dcount_width       => 9,
              c_enable_rlocs       => 0,
              c_has_dcount         => 0,
              c_has_rd_ack         => 1,
              c_has_rd_err         => 0,
              c_has_wr_ack         => 1,
              c_has_wr_err         => 0,
              c_memory_type        => 1,
              c_ports_differ       => 0,
              c_rd_ack_low         => 0,
              c_read_data_width    => 36,
              c_read_depth         => (C_TEMAC_RXFIFO/64),
              c_rd_err_low         => 0,
              c_wr_ack_low         => 0,
              c_wr_err_low         => 0,
              c_write_data_width   => 36,
              c_write_depth        => (C_TEMAC_RXFIFO/64)
         )
    port map(
              CLK          => LLTemac_Clk,              -- in
              SINIT        => comboFifoReset,--LLTemac_Rst,              -- in
              DIN          => ip2RSFIFO_Data,        -- in
              WR_EN        => ip2RSFIFO_Wreq,        -- in
              RD_EN        => lL2RSFIFO_Rden,        -- in
              DOUT         => rsFIFO2LL_Data,        -- out
              FULL         => rsFIFO2IP_Full,        -- out
              EMPTY        => rsFIFO2LL_Empty,       -- out
              RD_ACK       => rsFIFO2LL_Rdack,       -- out
              WR_ACK       => rsFIFO2IP_Wrack,       -- out
              RD_ERR       => open,                  -- out
              WR_ERR       => open,                  -- out
              DATA_COUNT   => open                   -- out
         ); 

--    I_RX_STATUS_FIFO : fifo_generator_v3_2
--      generic map (
--	c_rd_freq                      => 1,
--	c_wr_response_latency          => 1,
--	c_has_srst                     => 1,
--	c_has_rd_data_count            => 0,
--	c_din_width                    => 36,
--	c_has_wr_data_count            => 0,
--	c_implementation_type          => 0,
--	c_family                       => C_FAMILY,
--	c_has_wr_rst                   => 0,
--	c_wr_freq                      => 1,
--	c_underflow_low                => 0,
--	c_has_meminit_file             => 0,
--	c_has_overflow                 => 0,
--	c_preload_latency              => 1,
--	c_dout_width                   => 36,
--	c_rd_depth                     => (C_TEMAC_RXFIFO/64),
--	c_default_value                => "BlankString",
--	c_mif_file_name                => "BlankString",
--	c_has_underflow                => 0,
--	c_has_rd_rst                   => 0,
--	c_has_almost_full              => 0,
--	c_has_rst                      => 0,
--	c_data_count_width             => 9,
--	c_has_wr_ack                   => 1,
--	c_wr_ack_low                   => 0,
--	c_common_clock                 => 1,
--	c_rd_pntr_width                => 9,
--	c_has_almost_empty             => 0,
--	c_rd_data_count_width          => 9,
--	c_enable_rlocs                 => 0,
--	c_wr_pntr_width                => 9,
--	c_overflow_low                 => 0,
--	c_prog_empty_type              => 0,
--	c_optimization_mode            => 0,
--	c_wr_data_count_width          => 9,
--	c_preload_regs                 => 0,
--	c_dout_rst_val                 => "0",
--	c_has_data_count               => 0,
--	c_prog_full_thresh_negate_val  => ((C_TEMAC_RXFIFO/64)-3),
--	c_wr_depth                     => (C_TEMAC_RXFIFO/64),
--	c_prog_empty_thresh_negate_val => 3,
--	c_prog_empty_thresh_assert_val => 2,
--	c_has_valid                    => 1,
--	c_init_wr_pntr_val             => 0,
--	c_prog_full_thresh_assert_val  => ((C_TEMAC_RXFIFO/64)-2),
--	c_use_fifo16_flags             => 0,
--	c_has_backup                   => 0,
--	c_valid_low                    => 0,
--	c_prim_fifo_type               => "512x36",
--	c_count_type                   => 0,
--	c_prog_full_type               => 0,
--	c_memory_type                  => 1)
--      port map (
--        clk          => LLTemac_Clk,     -- in
--        din          => ip2RSFIFO_Data,  -- in
--        rd_en        => lL2RSFIFO_Rden,  -- in
--        SRST         => comboFifoReset,  -- in  
--        Wr_en        => ip2RSFIFO_Wreq,  -- in 
--        Dout         => rsFIFO2LL_Data,  -- out
--        Empty        => rsFIFO2LL_Empty, -- out
--        Full         => rsFIFO2IP_Full,  -- out
--        VALID        => rsFIFO2LL_Rdack, -- out
--        WR_ACK       => rsFIFO2IP_Wrack  -- out 
--        );
--
--    I_RX_STATUS_FIFO : fifo_generator_v4_2
--      generic map (
--	c_has_int_clk                  => 0,
--	c_rd_freq                      => 1,
--	c_wr_response_latency          => 1,
--	c_has_srst                     => 1,
--	c_has_rd_data_count            => 0,
--	c_din_width                    => 36,
--	c_has_wr_data_count            => 0,
--	c_full_flags_rst_val           => 0,
--	c_implementation_type          => 0,
--	c_family                       => C_FAMILY,
--	c_use_embedded_reg             => 0,
--	c_has_wr_rst                   => 0,
--	c_wr_freq                      => 1,
--	c_use_dout_rst                 => 1,
--	c_underflow_low                => 0,
--	c_has_meminit_file             => 0,
--	c_has_overflow                 => 0,
--	c_preload_latency              => 1,
--	c_dout_width                   => 36,
--	c_rd_depth                     => (C_TEMAC_RXFIFO/64),
--	c_default_value                => "BlankString",
--	c_mif_file_name                => "BlankString",
--	c_has_underflow                => 0,
--	c_has_rd_rst                   => 0,
--	c_has_almost_full              => 0,
--	c_has_rst                      => 0,
--	c_data_count_width             => 9,
--	c_has_wr_ack                   => 1,
--	c_use_ecc                      => 0,
--	c_wr_ack_low                   => 0,
--	c_common_clock                 => 1,
--	c_rd_pntr_width                => 9,
--	c_use_fwft_data_count          => 0,
--	c_has_almost_empty             => 0,
--	c_rd_data_count_width          => 9,
--	c_enable_rlocs                 => 0,
--	c_wr_pntr_width                => 9,
--	c_overflow_low                 => 0,
--	c_prog_empty_type              => 0,
--	c_optimization_mode            => 0,
--	c_wr_data_count_width          => 9,
--	c_preload_regs                 => 0,
--	c_dout_rst_val                 => "0",
--	c_has_data_count               => 0,
--	c_prog_full_thresh_negate_val  => ((C_TEMAC_RXFIFO/64)-3),
--	c_wr_depth                     => (C_TEMAC_RXFIFO/64),
--	c_prog_empty_thresh_negate_val => 3,
--	c_prog_empty_thresh_assert_val => 2,
--	c_has_valid                    => 1,
--	c_init_wr_pntr_val             => 0,
--	c_prog_full_thresh_assert_val  => ((C_TEMAC_RXFIFO/64)-2),
--	c_use_fifo16_flags             => 0,
--	c_has_backup                   => 0,
--	c_valid_low                    => 0,
--	c_prim_fifo_type               => "512x36",
--	c_count_type                   => 0,
--	c_prog_full_type               => 0,
--	c_memory_type                  => 1)
--      port map (
--        clk          => LLTemac_Clk,     -- in
--        din          => ip2RSFIFO_Data,  -- in
--        rd_en        => lL2RSFIFO_Rden,  -- in
--        SRST         => comboFifoReset,  -- in  
--        Wr_en        => ip2RSFIFO_Wreq,  -- in 
--        Dout         => rsFIFO2LL_Data,  -- out
--        Empty        => rsFIFO2LL_Empty, -- out
--        Full         => rsFIFO2IP_Full,  -- out
--        VALID        => rsFIFO2LL_Rdack, -- out
--        WR_ACK       => rsFIFO2IP_Wrack  -- out 
--        );
--
-- Start Async FIFO 6.1 model --------------------------------------                                                   

-- I_RX_STATUS_FIFO : async_fifo_v6_1 
--    generic map(
--         C_DATA_WIDTH       => 36,
--         C_ENABLE_RLOCS     =>  0,
--         C_FIFO_DEPTH       =>  (C_TEMAC_RXFIFO/64),
--         C_HAS_ALMOST_EMPTY =>  0,
--         C_HAS_ALMOST_FULL  =>  0,
--         C_HAS_RD_ACK       =>  1,
--         C_HAS_RD_COUNT     =>  0,
--         C_HAS_RD_ERR       =>  0,
--         C_HAS_WR_ACK       =>  1,
--         C_HAS_WR_COUNT     =>  0,
--         C_HAS_WR_ERR       =>  0,
--         C_RD_ACK_LOW       =>  0,
--         C_RD_COUNT_WIDTH   =>  2,
--         C_RD_ERR_LOW       =>  0,
--         C_USE_BLOCKMEM     =>  1,
--         C_WR_ACK_LOW       =>  0,
--         C_WR_COUNT_WIDTH   =>  2,
--         C_WR_ERR_LOW       =>  0   
--         )
--    port map(
--         Din            => ip2RSFIFO_Data,        -- in 
--         Wr_en          => ip2RSFIFO_Wreq,        -- in 
--         Wr_clk         => LLTemac_Clk,              -- in 
--         Rd_en          => lL2RSFIFO_Rden,        -- in 
--         Rd_clk         => LLTemac_Clk,              -- in 
--         Ainit          => LLTemac_Rst,              -- in 
--         Dout           => rsFIFO2LL_Data,        -- out
--         Full           => rsFIFO2IP_Full,        -- out
--         Empty          => rsFIFO2LL_Empty,       -- out
--         Almost_full    => open,                  -- out
--         Almost_empty   => open,                  -- out
--         Wr_count       => open,                  -- out                           
--         Rd_count       => open,                  -- out                            
--         Rd_ack         => rsFIFO2LL_Rdack,       -- out 
--         Rd_err         => open,                  -- out 
--         Wr_ack         => rsFIFO2IP_Wrack,       -- out 
--         Wr_err         => open                   -- out 
--         ); 


 I_RX_FIFO : sync_fifo_v5_0
    generic map(
              c_dcount_width       => 9,
              c_enable_rlocs       => 0,
              c_has_dcount         => 0,
              c_has_rd_ack         => 1,
              c_has_rd_err         => 0,
              c_has_wr_ack         => 1,
              c_has_wr_err         => 0,
              c_memory_type        => 1,
              c_ports_differ       => 0,
              c_rd_ack_low         => 0,
              c_read_data_width    => 36,
              c_read_depth         => (C_TEMAC_RXFIFO/4),
              c_rd_err_low         => 0,
              c_wr_ack_low         => 0,
              c_wr_err_low         => 0,
              c_write_data_width   => 36,
              c_write_depth        => (C_TEMAC_RXFIFO/4)
         )
    port map(
              CLK          => LLTemac_Clk,              -- in
              SINIT        => comboFifoReset,--LLTemac_Rst,              -- in
              DIN          => ip2RXFIFO_Data,        -- in
              WR_EN        => ip2RXFIFO_Wreq,        -- in
              RD_EN        => lL2RXFIFO_Rden,        -- in
              DOUT         => rxFIFO2LL_Data,        -- out
              FULL         => rxFIFO2IP_Full,        -- out
              EMPTY        => rxFIFO2LL_Empty,       -- out
              RD_ACK       => rxFIFO2LL_Rdack,       -- out
              WR_ACK       => rxFIFO2IP_Wrack,       -- out
              RD_ERR       => open,                  -- out
              WR_ERR       => open,                  -- out
              DATA_COUNT   => open                   -- out
         ); 

--    I_RX_FIFO : fifo_generator_v3_2
--      generic map (
--	c_rd_freq                      => 1,
--	c_wr_response_latency          => 1,
--	c_has_srst                     => 1,
--	c_has_rd_data_count            => 0,
--	c_din_width                    => 36,
--	c_has_wr_data_count            => 0,
--	c_implementation_type          => 0,
--	c_family                       => C_FAMILY,
--	c_has_wr_rst                   => 0,
--	c_wr_freq                      => 1,
--	c_underflow_low                => 0,
--	c_has_meminit_file             => 0,
--	c_has_overflow                 => 0,
--	c_preload_latency              => 1,
--	c_dout_width                   => 36,
--	c_rd_depth                     => (C_TEMAC_RXFIFO/4),
--	c_default_value                => "BlankString",
--	c_mif_file_name                => "BlankString",
--	c_has_underflow                => 0,
--	c_has_rd_rst                   => 0,
--	c_has_almost_full              => 0,
--	c_has_rst                      => 0,
--	c_data_count_width             => 9,
--	c_has_wr_ack                   => 1,
--	c_wr_ack_low                   => 0,
--	c_common_clock                 => 1,
--	c_rd_pntr_width                => 9,
--	c_has_almost_empty             => 0,
--	c_rd_data_count_width          => 9,
--	c_enable_rlocs                 => 0,
--	c_wr_pntr_width                => 9,
--	c_overflow_low                 => 0,
--	c_prog_empty_type              => 0,
--	c_optimization_mode            => 0,
--	c_wr_data_count_width          => 9,
--	c_preload_regs                 => 0,
--	c_dout_rst_val                 => "0",
--	c_has_data_count               => 0,
--	c_prog_full_thresh_negate_val  => ((C_TEMAC_RXFIFO/4)-3),
--	c_wr_depth                     => (C_TEMAC_RXFIFO/4),
--	c_prog_empty_thresh_negate_val => 3,
--	c_prog_empty_thresh_assert_val => 2,
--	c_has_valid                    => 1,
--	c_init_wr_pntr_val             => 0,
--	c_prog_full_thresh_assert_val  => ((C_TEMAC_RXFIFO/4)-2),
--	c_use_fifo16_flags             => 0,
--	c_has_backup                   => 0,
--	c_valid_low                    => 0,
--	c_prim_fifo_type               => "512x36",
--	c_count_type                   => 0,
--	c_prog_full_type               => 0,
--	c_memory_type                  => 1)
--      port map (
--        clk          => LLTemac_Clk,     -- in
--        din          => ip2RXFIFO_Data,  -- in
--        rd_en        => lL2RXFIFO_Rden,  -- in
--        SRST         => comboFifoReset,  -- in  
--        Wr_en        => ip2RXFIFO_Wreq,  -- in 
--        Dout         => rxFIFO2LL_Data,  -- out
--        Empty        => rxFIFO2LL_Empty, -- out
--        Full         => rxFIFO2IP_Full,  -- out
--        VALID        => rxFIFO2LL_Rdack, -- out
--        WR_ACK       => rxFIFO2IP_Wrack  -- out 
--        );
--
--    I_RX_FIFO : fifo_generator_v4_2
--      generic map (
--	c_has_int_clk                  => 0,
--	c_rd_freq                      => 1,
--	c_wr_response_latency          => 1,
--	c_has_srst                     => 1,
--	c_has_rd_data_count            => 0,
--	c_din_width                    => 36,
--	c_has_wr_data_count            => 0,
--	c_full_flags_rst_val           => 0,
--	c_implementation_type          => 0,
--	c_family                       => C_FAMILY,
--	c_use_embedded_reg             => 0,
--	c_has_wr_rst                   => 0,
--	c_wr_freq                      => 1,
--	c_use_dout_rst                 => 1,
--	c_underflow_low                => 0,
--	c_has_meminit_file             => 0,
--	c_has_overflow                 => 0,
--	c_preload_latency              => 1,
--	c_dout_width                   => 36,
--	c_rd_depth                     => (C_TEMAC_RXFIFO/4),
--	c_default_value                => "BlankString",
--	c_mif_file_name                => "BlankString",
--	c_has_underflow                => 0,
--	c_has_rd_rst                   => 0,
--	c_has_almost_full              => 0,
--	c_has_rst                      => 0,
--	c_data_count_width             => 9,
--	c_has_wr_ack                   => 1,
--	c_use_ecc                      => 0,
--	c_wr_ack_low                   => 0,
--	c_common_clock                 => 1,
--	c_rd_pntr_width                => 9,
--	c_use_fwft_data_count          => 0,
--	c_has_almost_empty             => 0,
--	c_rd_data_count_width          => 9,
--	c_enable_rlocs                 => 0,
--	c_wr_pntr_width                => 9,
--	c_overflow_low                 => 0,
--	c_prog_empty_type              => 0,
--	c_optimization_mode            => 0,
--	c_wr_data_count_width          => 9,
--	c_preload_regs                 => 0,
--	c_dout_rst_val                 => "0",
--	c_has_data_count               => 0,
--	c_prog_full_thresh_negate_val  => ((C_TEMAC_RXFIFO/4)-3),
--	c_wr_depth                     => (C_TEMAC_RXFIFO/4),
--	c_prog_empty_thresh_negate_val => 3,
--	c_prog_empty_thresh_assert_val => 2,
--	c_has_valid                    => 1,
--	c_init_wr_pntr_val             => 0,
--	c_prog_full_thresh_assert_val  => ((C_TEMAC_RXFIFO/4)-2),
--	c_use_fifo16_flags             => 0,
--	c_has_backup                   => 0,
--	c_valid_low                    => 0,
--	c_prim_fifo_type               => "512x36",
--	c_count_type                   => 0,
--	c_prog_full_type               => 0,
--	c_memory_type                  => 1)
--      port map (
--        clk          => LLTemac_Clk,     -- in
--        din          => ip2RXFIFO_Data,  -- in
--        rd_en        => lL2RXFIFO_Rden,  -- in
--        SRST         => comboFifoReset,  -- in  
--        Wr_en        => ip2RXFIFO_Wreq,  -- in 
--        Dout         => rxFIFO2LL_Data,  -- out
--        Empty        => rxFIFO2LL_Empty, -- out
--        Full         => rxFIFO2IP_Full,  -- out
--        VALID        => rxFIFO2LL_Rdack, -- out
--        WR_ACK       => rxFIFO2IP_Wrack  -- out 
--        );
--
-- Start Async FIFO 6.1 model --------------------------------------                                                   

-- I_RX_FIFO : async_fifo_v6_1 
--    generic map(
--         C_DATA_WIDTH       =>  36,
--         C_ENABLE_RLOCS     =>  0,
--         C_FIFO_DEPTH       =>  (C_TEMAC_RXFIFO/4),
--         C_HAS_ALMOST_EMPTY =>  0,
--         C_HAS_ALMOST_FULL  =>  1,
--         C_HAS_RD_ACK       =>  1,
--         C_HAS_RD_COUNT     =>  0,
--         C_HAS_RD_ERR       =>  0,
--         C_HAS_WR_ACK       =>  1,
--         C_HAS_WR_COUNT     =>  0,
--         C_HAS_WR_ERR       =>  0,
--         C_RD_ACK_LOW       =>  0,
--         C_RD_COUNT_WIDTH   =>  2,
--         C_RD_ERR_LOW       =>  0,
--         C_USE_BLOCKMEM     =>  1,
--         C_WR_ACK_LOW       =>  0,
--         C_WR_COUNT_WIDTH   =>  2,
--         C_WR_ERR_LOW       =>  0   
--         )
--    port map(
--         Din            => ip2RXFIFO_Data,         -- in 
--         Wr_en          => ip2RXFIFO_Wreq,         -- in 
--         Wr_clk         => LLTemac_Clk,               -- in 
--         Rd_en          => lL2RXFIFO_Rden,         -- in 
--         Rd_clk         => LLTemac_Clk,               -- in 
--         Ainit          => LLTemac_Rst,               -- in 
--         Dout           => rxFIFO2LL_Data,         -- out
--         Full           => rxFIFO2IP_Full,         -- out
--         Empty          => rxFIFO2LL_Empty,        -- out
--         Almost_full    => rxFIFO2IP_Almost_Full,  -- out
--         Almost_empty   => open,                   -- out
--         Wr_count       => open,                   -- out                           
--         Rd_count       => open,                   -- out                            
--         Rd_ack         => rxFIFO2LL_Rdack,        -- out 
--         Rd_err         => open,                   -- out 
--         Wr_ack         => rxFIFO2IP_Wrack,        -- out 
--         Wr_err         => open                    -- out 
--         ); 

full_c <= '1' when (occupancy_i = fullMask) else '0';
afull_c <= '1' when (occupancy_i = afullMask) else '0';
empty_c <= '1' when (occupancy_i = emptyMask) else '0';
aempty_c <= '1' when (occupancy_i = aemptyMask) else '0';

rxFIFO2IP_Almost_Full <= afull_c or afull_d or afull_d1 or afull_d2 or afull_d3 or afull_d4 or afull_d5 or afull_d6 or full_c;        

FLAGS : process(LLTemac_Clk)
   begin
      if(rising_edge(LLTemac_Clk)) then
         if(comboFifoReset='1') then
            occupancy_i <= (others => '0');
         elsif(rxFIFO2IP_Wrack='1' and rxFIFO2LL_Rdack='1') then
            occupancy_i <= occupancy_i;
         elsif(rxFIFO2IP_Wrack='1') then
            occupancy_i <= occupancy_i+1;
         elsif(rxFIFO2LL_Rdack='1') then
            occupancy_i <= occupancy_i-1;
         else
            occupancy_i <= occupancy_i;         
         end if;
      end if;
      if(rising_edge(LLTemac_Clk)) then
         if(LLTemac_Rst='1') then
            full_d   <= '0';
            empty_d  <= '1';
            afull_d  <= '0';
            afull_d1 <= '0';
            afull_d2 <= '0';
            afull_d3 <= '0';
            afull_d4 <= '0';
            afull_d5 <= '0';
            afull_d6 <= '0';
            aempty_d <= '0';
         else
            full_d   <= full_c;
            empty_d  <= empty_c;
            afull_d  <= afull_c;
            afull_d1 <= afull_d;
            afull_d2 <= afull_d1;
            afull_d3 <= afull_d2;
            afull_d4 <= afull_d3;
            afull_d5 <= afull_d4;
            afull_d6 <= afull_d5;
            aempty_d <= aempty_c;
         end if;
      end if;
   end process;

I_RX_TEMAC_IF : entity xps_ll_temac_v1_01_b.rx_temac_if 
   generic map(
      C_FAMILY          => C_FAMILY,
      C_RESET_ACTIVE    => C_RESET_ACTIVE,
      C_BUS_DWIDTH      => C_BUS_DWIDTH,
      C_TEMAC_TYPE      =>  C_TEMAC_TYPE,
      C_TEMAC_RXCSUM    => C_TEMAC_RXCSUM,
      C_CLIENT_DWIDTH   => C_CLIENT_DWIDTH,
      C_RX_LEN_LENGTH   => RX_LEN_LENGTH
      )

   port map(
      LocalLinkPaused          => localLinkPaused,          -- in
      PauseLocalLinkFull       => pauseLocalLinkFull,       -- out 
      FifoFullReset            => fifoFullReset,            -- out 
      LLTemac_Clk              => LLTemac_Clk,              -- in  
      LLTemac_Rst              => LLTemac_Rst,              -- in  
      RSFIFO2IP_Wrack          => rsFIFO2IP_Wrack,          -- in  
      RSFIFO2IP_Full           => rsFIFO2IP_Full,           -- in  
      RSFIFO2IP_Ovr            => rsFIFO2IP_Ovr,            -- in  
      RSFIFO2LL_Und            => rsFIFO2LL_Und,            -- in  
      IP2RSFIFO_Wreq           => ip2RSFIFO_Wreq,           -- out 
      IP2RSFIFO_Data           => ip2RSFIFO_Data,           -- out 
      RXFIFO2IP_Wrack          => rxFIFO2IP_Wrack,          -- in  
      RXFIFO2IP_Almost_Full    => rxFIFO2IP_Almost_Full,    -- in  
      RXFIFO2IP_Full           => rxFIFO2IP_Full,           -- in  
      
      RXFIFO2IP_Ovr            => rxFIFO2IP_Ovr,            -- in  
      RXFIFO2LL_Und            => rxFIFO2LL_Und,            -- in  
      IP2RXFIFO_Wreq           => ip2RXFIFO_Wreq,           -- out 
      Ip2RXFIFO_Wreq_unqualified=> ip2RXFIFO_Wreq_unqualified,           -- out 
      IP2RXFIFO_Data           => ip2RXFIFO_Data,           -- out 
      
      RegCR_BrdCast_Rej        => RegCR_BrdCast_Rej,        -- in  
      RegCR_MulCast_Rej        => RegCR_MulCast_Rej,        -- in  
      Pckt_Ovr_Run             => Pckt_Ovr_Run,             -- out 
      Rx_cmplt                 => rx_cmplt_i,               -- out 
      Rx_Pckt_Rej              => Rx_Pckt_Rej,              -- out 
      
      Rx_Cl_Clk                => Rx_Cl_Clk,                -- in  
      EmacClientRxBadFrame     => EmacClientRxBadFrame,     -- in  
      EmacClientRxd            => EmacClientRxd,            -- in  
      EmacClientRxdVld         => EmacClientRxdVld,         -- in  
      EmacClientRxFrameDrop    => EmacClientRxFrameDrop,    -- in  
      EmacClientRxGoodFrame    => EmacClientRxGoodFrame,    -- in  
      EmacClientRxStats        => EmacClientRxStats,        -- in  
      SoftEmacClientRxStats    =>  softEmacClientRxStats,   -- in
      EmacClientRxStatsVld     => EmacClientRxStatsVld,     -- in  
--      TakeRxStatsNow           => TakeRxStatsNow,         -- in  
      EmacClientRxStats_i      => EmacClientRxStats_i,      -- in  
      ClFifoOverflow           => ClFifoOverflow            -- out 
      );

end beh;
