------------------------------------------------------------------------------
-- $Id: tx_ll_if.vhd,v 1.3 2008/06/19 21:41:01 mwelter Exp $
------------------------------------------------------------------------------
-- tx_ll_if.vhd
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
-- **                                                                       **
-- **  This text contains proprietary, confidential                         **
-- **  information of Xilinx, Inc. , is distributed by                      **
-- **  under license from Xilinx, Inc., and may be used,                    **
-- **  copied and/or disclosed only pursuant to the terms                   **
-- **  of a valid license agreement with Xilinx, Inc.                       **
-- **                                                                       **
-- **  Unmodified source code is guaranteed to place and route,             **
-- **  function and run at speed according to the datasheet                 **
-- **  specification. Source code is provided "as-is", with no              **
-- **  obligation on the part of Xilinx to provide support.                 **
-- **                                                                       **
-- **  Xilinx Hotline support of source code IP shall only include          **
-- **  standard level Xilinx Hotline support, and will only address         **
-- **  issues and questions related to the standard released Netlist        **
-- **  version of the core (and thus indirectly, the original core source). **
-- **                                                                       **
-- **  The Xilinx Support Hotline does not have access to source            **
-- **  code and therefore cannot answer specific questions related          **
-- **  to source HDL. The Xilinx Support Hotline will only be able          **
-- **  to confirm the problem in the Netlist version of the core.           **
-- **                                                                       **
-- **  This copyright and support notice must be retained as part           **
-- **  of this text at all times.                                           **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:        tx_ll_if.vhd
-- Version:         v1.00a
-- Description:     interface block between LL and hard fifos
--
------------------------------------------------------------------------------
-- Structure:   This section should show the hierarchical structure of the
--              designs. Separate lines with blank lines if necessary to improve
--              readability.
--
--              top_level.vhd
--                  -- second_level_file1.vhd
--                      -- third_level_file1.vhd
--                          -- fourth_level_file.vhd
--                      -- third_level_file2.vhd
--                  -- second_level_file2.vhd
--                  -- second_level_file3.vhd
--
--              This section is optional for common/shared modules but should
--              contain a statement stating it is a common/shared module.
------------------------------------------------------------------------------
-- Author:      DRP
-- History:
--  DRP      2006.04.24      -- First version
--
--  <initials>      <date>
-- ^^^^^^
--      Description of changes. If multiple lines are needed to fully describe
--      the changes made to the design, these lines should align with each other.
-- ~~~~~~
--
--  <initials>      <date>
-- ^^^^^^
--      More changes
-- ~~~~~~
--
------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x"
--      reset signals:                          "rst", "rst_n"
--      generics:                               "C_*"
--      user defined types:                     "*_TYPE"
--      state machine next state:               "*_ns"
--      state machine current state:            "*_cs"
--      combinatorial signals:                  "*_cmb"
--      pipelined or register delay signals:    "*_d#"
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce"
--      internal version of : out   std_logic; port         "*_i"
--      device pins:                            "*_pin"
--      ports:                                  - Names begin with Uppercase
--      processes:                              "*_PROCESS"
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;


library unisim;
use unisim.vcomponents.all;

library xps_ll_temac_v1_01_b;
use xps_ll_temac_v1_01_b.all;

------------------------------------------------------------------------------
-- Port Declaration
------------------------------------------------------------------------------

entity tx_ll_if is
   generic (
     C_TEMAC_TXCSUM        : integer := 0;  
     C_FIFO_DEPTH_LOG2X    : integer := 13
     );
   port(
      LLTemac_Clk         : in  std_logic;
      LLTemac_Rst         : in  std_logic;
      
      LLTemac_Data        : in  std_logic_vector(0 to 31);
      LLTemac_SOF_n       : in  std_logic;
      LLTemac_SOP_n       : in  std_logic;
      LLTemac_EOF_n       : in  std_logic;
      LLTemac_EOP_n       : in  std_logic;
      LLTemac_SRC_RDY_n   : in  std_logic;
      LLTemac_REM         : in  std_logic_vector(0 to 3);
      LLTemac_DST_RDY_n   : in  std_logic;

      CSFIFO2LL_full      : in  std_logic;
      LL2CSFIFO_wren      : out std_logic;
      LL2CSFIFO_data      : out std_logic_vector(0 to 35);
      
      TXFIFO2LL_full      : in  std_logic;      
      LL2TXFIFO_data      : out std_logic_vector(0 to 35);
      LL2TXFIFO_wren      : out std_logic;
      Csum_Ready          : in  std_logic
      );
end tx_ll_if;

------------------------------------------------------------------------------
-- Definition of Generics:
--
-- Definition of Ports:
--
------------------------------------------------------------------------------

architecture beh of tx_ll_if is

------------------------------------------------------------------------------
-- Signal Declarations
------------------------------------------------------------------------------

signal sop                 : std_logic;
signal eop                 : std_logic;
signal csum_insert         : std_logic_vector(0 to 15);
signal csum_data           : std_logic_vector(0 to 15);
signal csum_en             : std_logic;
signal csum_wren           : std_logic;
signal wren                : std_logic;
signal lL2TXFIFO_Wren_i    : std_logic;
signal rem_encoded         : std_logic_vector(0 to 1);
signal tx_pckt_valid       : std_logic;
signal tx_pckt_valid_i     : std_logic;
signal header              : std_logic;
signal tied_low            : std_logic_vector(0 to 31) := (others=>'0');
signal remainder           : integer;
signal lLTemac_Data_i      : std_logic_vector(0 to 31);

signal x_header            : std_logic;
signal x_rem_encoded       : std_logic_vector(0 to 1);
signal x_sop               : std_logic;
signal x_eop               : std_logic;
signal x_llTemac_Data      : std_logic_vector(0 to 31);
signal x_ll2txfifo_wren    : std_logic;
signal x_LLTemac_DST_RDY_n : std_logic;
signal x_LLTemac_SRC_RDY_n : std_logic;
signal x_LLTemac_EOF_n     : std_logic;
signal x_LL2TXFIFO_Data    : std_logic_vector(0 to 35);

signal lLTemac_SOF_n_d1    : std_logic;
signal lLTemac_SOF_n_d2    : std_logic;
signal lLTemac_SOF_n_d3    : std_logic;
signal lLTemac_SOF_n_d4    : std_logic;
signal lLTemac_SOF_n_d5    : std_logic;
signal lLTemac_SOF_n_d6    : std_logic;
signal lLTemac_SOF_n_d7    : std_logic;
signal lLTemac_SOF_n_d8    : std_logic;
signal lLTemac_SOF_n_d9    : std_logic;
signal new_header          : std_logic;
signal new_rem_encoded     : std_logic_vector(0 to 1);
signal new_lL2TXFIFO_Wren_i: std_logic;

begin

------------------------------------------------------------------------------
-- Concurrent assignments
------------------------------------------------------------------------------

LL2CSFIFO_Data <= csum_data&csum_insert&csum_en&tied_low(0 to 2);
LL2CSFIFO_Wren <= csum_wren; 
                     
new_lL2TXFIFO_Wren_i <= not TXFIFO2LL_Full and 
                     not LLTemac_DST_RDY_n and
                     not LLTemac_SRC_RDY_n and
                     LLTemac_EOF_n and
                     not new_header;
LL2TXFIFO_Wren <= new_lL2TXFIFO_Wren_i;
LL2TXFIFO_Data <= LLTemac_Data&new_rem_encoded&sop&eop;

sop <= not LLTemac_DST_RDY_n and not LLTemac_SRC_RDY_n and not LLTemac_SOP_n;
eop <= not LLTemac_DST_RDY_n and not LLTemac_SRC_RDY_n and not LLTemac_EOP_n;


------------------------------------------------------------------------------
-- This process detects the header
------------------------------------------------------------------------------



NEW_HEADER_VALID : process(LLTemac_Clk)
   begin
      if(rising_edge(LLTemac_Clk))then
         if(LLTemac_Rst='1') then
            lLTemac_SOF_n_d1 <= '0';
            lLTemac_SOF_n_d2 <= '0';
            lLTemac_SOF_n_d3 <= '0';
            lLTemac_SOF_n_d4 <= '0';
            lLTemac_SOF_n_d5 <= '0';
            lLTemac_SOF_n_d6 <= '0';
            lLTemac_SOF_n_d7 <= '0';
            lLTemac_SOF_n_d8 <= '0';
            lLTemac_SOF_n_d9 <= '0';
         else
            lLTemac_SOF_n_d1 <= LLTemac_SOF_n;
            lLTemac_SOF_n_d2 <= lLTemac_SOF_n_d1;
            lLTemac_SOF_n_d3 <= lLTemac_SOF_n_d2;
            lLTemac_SOF_n_d4 <= lLTemac_SOF_n_d3;
            lLTemac_SOF_n_d5 <= lLTemac_SOF_n_d4;
            lLTemac_SOF_n_d6 <= lLTemac_SOF_n_d5;
            lLTemac_SOF_n_d7 <= lLTemac_SOF_n_d6;
            lLTemac_SOF_n_d8 <= lLTemac_SOF_n_d7;
            lLTemac_SOF_n_d9 <= lLTemac_SOF_n_d8;
         end if;
      end if;
   end process;
   new_header <= not(LLTemac_SOF_n and lLTemac_SOF_n_d1 and lLTemac_SOF_n_d2 and lLTemac_SOF_n_d3 and lLTemac_SOF_n_d4 and lLTemac_SOF_n_d5 and lLTemac_SOF_n_d6 and lLTemac_SOF_n_d7 and lLTemac_SOF_n_d8 and lLTemac_SOF_n_d9);
------------------------------------------------------------------------------
-- This process extracts the csum, csum offset, csum enable and strips
-- invalid remainder bytes
------------------------------------------------------------------------------

   
TX_PACKET_VALID : process(LLTemac_Clk)
   begin
      if(rising_edge(LLTemac_Clk))then
         if(LLTemac_Rst='1' or LLTemac_EOP_n='0') then
            tx_pckt_valid_i <= '0';
         elsif(LLTemac_SOP_n='0')then
            tx_pckt_valid_i <= '1';
         end if;
      end if;
   end process;
   
tx_pckt_valid <= tx_pckt_valid_i or not LLTemac_SOP_n; 


remainder  <= to_integer(unsigned(LLTemac_REM));



REMAINDER_PROCESS : process(LLTemac_Rst, LLTemac_REM, LLTemac_Data,
                            LLTemac_EOP_n, tx_pckt_valid, remainder,
                            LLTemac_SRC_RDY_n, LLTemac_DST_RDY_n, Csum_Ready)
   begin
      if(LLTemac_Rst='1') then
         lLTemac_Data_i <= LLTemac_Data;--(others=>'0');       
      else--if(Csum_Ready='1') then
         case remainder is
            when 0 =>
               lLTemac_Data_i <= LLTemac_Data;
                  
            when 1 => 
               lLTemac_Data_i <= LLTemac_Data(0 to 23)&tied_low(24 to 31);
                  
            when 3 => 
               lLTemac_Data_i <= LLTemac_Data(0 to 15)&tied_low(16 to 31);
                  
            when 7 => 
               lLTemac_Data_i <= LLTemac_Data(0 to 7)&tied_low(8 to 31);
              
            when others => 
               lLTemac_Data_i <= LLTemac_Data;--null;
         end case;
--      else
--         lLTemac_Data_i <= LLTemac_Data;--(others=>'0');

      end if;
   end process;
       
                  
         
GEN_TXCSUM : if(C_TEMAC_TXCSUM = 1) generate   





      
   I_TXCSUM : entity xps_ll_temac_v1_01_b.tx_csum_top_wrapper
   generic map(
      C_FIFO_DEPTH_LOG2X => C_FIFO_DEPTH_LOG2X
      )
   port map(
      Clk                  => LLTemac_Clk,         -- in  
      Rst                  => LLTemac_Rst,         -- in  
      Din                  => lLTemac_Data_i,      -- in  
      Wren                 => new_lL2TXFIFO_Wren_i,    -- in 
      LLTemac_SOF_n        => LLTemac_SOF_n,       -- in
      LLTemac_EOP_n        => LLTemac_EOP_n,       -- in
      LLTemac_SRC_RDY_n    => LLTemac_SRC_RDY_n,   -- in
      LLTemac_DST_RDY_n    => LLTemac_DST_RDY_n,   -- in
      Csum_insert          => csum_insert,         -- out 
      Csum_en              => csum_en,             -- out 
      Csum_wren            => csum_wren,           -- out 
      Csum_data            => csum_data            -- out      
      );
end generate;


GEN_NO_TXCSUM : if(C_TEMAC_TXCSUM = 0) generate

   csum_data <= (others=>'0');
   csum_insert <= (others=>'0');
   csum_en <= '0';
   wren <= '0';
   
   csum_wren <= not LLTemac_DST_RDY_n and 
                not LLTemac_SRC_RDY_n and
                not LLTemac_EOP_n;
end generate;
                
      
ENCODE_REM_PROCESS : process(LLTemac_Rst, LLTemac_REM)
   begin
      if(LLTemac_Rst='1') then
         new_rem_encoded <= (others=>'0');       
      else
         case LLTemac_REM(0 to 3) is
            when "0111" =>
               new_rem_encoded <= "00";                  
            when "0011" => 
               new_rem_encoded <= "01";                 
            when "0001" => 
               new_rem_encoded <= "10";                 
            when "0000" => 
               new_rem_encoded <= "11";
            when others => 
               new_rem_encoded <= "00";
         end case;
      end if;
   end process;

end beh;
