#include "ethernet.h"
#include "xparameters.h"        /* defines XPAR values */
#include "xlltemac.h"           /* defines XLlTemac API */
#include "xllfifo.h"
#include "xexception_l.h"
#include "xintc.h"

#define TEMAC_DEVICE_ID   XPAR_LLTEMAC_0_DEVICE_ID
#define FIFO_DEVICE_ID    XPAR_LLFIFO_0_DEVICE_ID

XLlTemac TemacInstance;  /* Temac device instance used throughout examples */
XLlFifo FifoInstance;    /* Fifo device instance used throughout examples */

void RxTest(void);

void TemacUtilErrorTrap(char *Message)
{
	xil_printf("%s\r\n", Message);
}

int main()
{
	int Status, loop;
	XLlTemac_Config *MacCfgPtr;
	u32 Rdy;
	u16 u16param;
	
	xil_printf("EMAC Test v0 - Entering Main\r\n");

	MacCfgPtr = XLlTemac_LookupConfig(TEMAC_DEVICE_ID);
	
	Status = XLlTemac_CfgInitialize(&TemacInstance, MacCfgPtr,
									MacCfgPtr->BaseAddress);

	xil_printf("Temac Cfg:\r\n");
	xil_printf("\tStatus: %d\r\n", Status);
	xil_printf("\tBaseAddress: 0x%x\r\n", MacCfgPtr->BaseAddress);
	xil_printf("\tPhyType: %d\r\n", MacCfgPtr->PhyType);

	if (Status != XST_SUCCESS) {
			TemacUtilErrorTrap("Error in initialize");
			return XST_FAILURE;
	}

//	XLlTemac_Reset(&TemacInstance, XTE_RESET_HARD);

	Status = XLlTemac_ReadReg(TemacInstance.Config.BaseAddress, XTE_IS_OFFSET) & XTE_INT_MGT_LOCK_MASK;
	xil_printf("MGT Lock Status: %d\r\n", Status);

   	Status  = XLlTemac_ClearOptions(&TemacInstance, XTE_LENTYPE_ERR_OPTION | XTE_FLOW_CONTROL_OPTION);// | XTE_MULTICAST_OPTION);
	Status |= XLlTemac_SetOptions(&TemacInstance, XTE_PROMISC_OPTION | XTE_MULTICAST_OPTION | XTE_BROADCAST_OPTION | XTE_RECEIVER_ENABLE_OPTION | XTE_TRANSMITTER_ENABLE_OPTION); //| XTE_JUMBO_OPTION
	if (Status != XST_SUCCESS)
		xil_printf("Error setting EMAC options\r\n, code %d", Status);


	XLlFifo_Initialize(&FifoInstance, XLlTemac_LlDevBaseAddress(&TemacInstance));

	xil_printf("Did LLFifo CfgInit\r\n");

	if (!XLlTemac_IsFifo(&TemacInstance)) {
			TemacUtilErrorTrap("Device HW not configured for FIFO direct mode\r\n");
			return XST_FAILURE;
	}

	/* Make sure the hard temac is ready */
	Status = XLlTemac_ReadReg(TemacInstance.Config.BaseAddress, XTE_RDY_OFFSET);
	while ((Status & XTE_RDY_HARD_ACS_RDY_MASK) == 0)
	{
		Status = XLlTemac_ReadReg(TemacInstance.Config.BaseAddress, XTE_RDY_OFFSET);
	}

	XLlTemac_SetOperatingSpeed(&TemacInstance, 1000);
	sleep(3);

	/* Make sure the hard temac is ready */
	Status = XLlTemac_ReadReg(TemacInstance.Config.BaseAddress, XTE_RDY_OFFSET);
	while ((Status & XTE_RDY_HARD_ACS_RDY_MASK) == 0)
	{
		Status = XLlTemac_ReadReg(TemacInstance.Config.BaseAddress, XTE_RDY_OFFSET);
	}

    XLlTemac_Start(&TemacInstance);

	xil_printf("Temac started? %d\r\n", XLlTemac_IsStarted(&TemacInstance));
	xil_printf("Temac status: 0x%x\r\n", XLlTemac_Status(&TemacInstance));
	xil_printf("Starting RxTest\r\n");
	RxTest();

	xil_printf("Exiting...\r\n");

	return 0;
}

#if 1
void RxTest(void)
{
	int Status;
	int TxFrameLength, RxFrameLength, PayloadSize;
	u32 FifoFreeBytes;
	int pktTxNum = 0;
	unsigned char RxFrame[1600] __attribute__ ((aligned(8)));		/* Receive frame buffer */

	PayloadSize = 1400;
	
	/************** Rx Test *********************/
    while(1)
    {
        /* Wait for a frame to arrive */
		Status = XST_NO_DATA;
        do
        {
			xil_printf("Temac status: 0x%x\r\n", XLlTemac_Status(&TemacInstance));
			/*
			 * Try to get status
			 * We could have polled the status bits instead. Either way works.
			 */
			if (XLlFifo_Status(&FifoInstance) & XLLF_INT_RC_MASK) {
				XLlFifo_IntClear(&FifoInstance, XLLF_INT_RC_MASK);
				Status = XST_SUCCESS;
				xil_printf("FIFO saw a pkt\r\n");
			}
			if (XLlFifo_Status(&FifoInstance) & XLLF_INT_ERROR_MASK) {
				XLlFifo_IntClear(&FifoInstance, XLLF_INT_ERROR_MASK);
				Status = XST_FIFO_ERROR;
				xil_printf("FIFO error\r\n");
				break;
			}
			if (XLlTemac_Status(&TemacInstance) & XTE_INT_RXRJECT_MASK) {
				xil_printf("Dropped frame! 3\r\n");
				XLlTemac_IntClear(&TemacInstance, (XTE_INT_RXFIFOOVR_MASK | XTE_INT_RXRJECT_MASK));
			}
			/* When the RXFIFOOVR bit is set, the RXRJECT bit also gets set */
			if (XLlTemac_Status(&TemacInstance) & XTE_INT_RXFIFOOVR_MASK) {
				xil_printf("Dropped frame! 4\r\n");
				XLlTemac_IntClear(&TemacInstance, (XTE_INT_RXFIFOOVR_MASK ));
			}
		} while (Status == XST_NO_DATA);
	
        if (Status == XST_SUCCESS)
        {
			/* While there is data in the fifo ... */
			while (XLlFifo_RxOccupancy(&FifoInstance)) {
				/* Receive frame */
				
				/* A frame is in the fifo */
				RxFrameLength = XLlFifo_RxGetLen(&FifoInstance);
				
				/* Read it out */
				XLlFifo_Read(&FifoInstance, &RxFrame, RxFrameLength);
				
				xil_printf("Got a pkt (len=%d)\r\n", RxFrameLength);
				
				if (XLlTemac_Status(&TemacInstance) & XTE_INT_RXRJECT_MASK) {
					xil_printf("Dropped frame! 1\r\n");
					XLlTemac_IntClear(&TemacInstance, (XTE_INT_RXFIFOOVR_MASK | XTE_INT_RXRJECT_MASK));
				}
				/* When the RXFIFOOVR bit is set, the RXRJECT bit also gets set */
				if (XLlTemac_Status(&TemacInstance) & XTE_INT_RXFIFOOVR_MASK) {
					xil_printf("Dropped frame 2!\r\n");
					XLlTemac_IntClear(&TemacInstance, (XTE_INT_RXFIFOOVR_MASK ));
				}
			}
		}
        /* Error, set flag to cause reset of device */
        else
        {
            xil_printf("Fatal error detected, code %d\r\n", XLlTemac_Status(&TemacInstance));
            break;
        }
    }
}
#endif
